#!/usr/bin/env node
"use strict";
// For large dictionaries, it is necessary to increase the memory limit.
// # !/usr/bin/env node --max_old_space_size=8192
Object.defineProperty(exports, "__esModule", { value: true });
const compiler_1 = require("./compiler");
const path = require("path");
const program = require("commander");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const glob = require("glob");
const npmPackage = require(path.join(__dirname, '..', 'package.json'));
const globRx = rxjs_1.bindNodeCallback(glob);
program
    .version(npmPackage.version);
program
    .command('compile <src...>')
    .description('compile words lists into simple dictionary files.')
    .option('-o, --output <path>', 'Specify the output directory, otherwise files are written back to the same location.')
    .option('-n, --no-compress', 'By default the files are Gzipped, this will turn that off.')
    .option('-s, --no-split', 'Treat each line as a dictionary entry, do not split')
    .option('--no-sort', 'Do not sort the result')
    .action((src, options) => {
    console.log('Compile:\n output: %s\n compress: %s\n files:\n  %s \n\n', options.output || 'default', options.compress ? 'true' : 'false', src.join('\n  '));
    const ext = '.txt' + (options.compress ? '.gz' : '');
    rxjs_1.from(src).pipe(operators_1.flatMap(src => globRx(src)), operators_1.flatMap(s => s), operators_1.map(s => {
        const outFilename = path.basename(s).replace(/(\.txt|\.dic|\.aff)?$/, ext);
        const dir = options.output ? options.output : path.dirname(s);
        return [s, path.join(dir, outFilename)];
    }), operators_1.concatMap(([src, dst]) => {
        console.log('Process "%s" to "%s"', src, dst);
        return compiler_1.compileWordList(src, dst, { splitWords: options.split, sort: options.sort }).then(() => src);
    }))
        .forEach(name => console.log(`Complete.`));
});
program
    .command('compile-trie <src...>')
    .description('Compile words lists or Hunspell dictionary into trie files used by cspell.')
    .option('-o, --output <path>', 'Specify the output directory, otherwise files are written back to the same location.')
    .option('-n, --no-compress', 'By default the files are Gzipped, this will turn that off.')
    .action((src, options) => {
    console.log('Compile:\n output: %s\n compress: %s\n files:\n  %s \n\n', options.output || 'default', options.compress ? 'true' : 'false', src.join('\n  '));
    const ext = '.trie' + (options.compress ? '.gz' : '');
    rxjs_1.from(src).pipe(operators_1.flatMap(src => globRx(src)), operators_1.flatMap(s => s), operators_1.map(s => {
        const outFilename = path.basename(s).replace(/(\.txt|\.dic|\.aff)?$/, ext);
        const dir = options.output ? options.output : path.dirname(s);
        return [s, path.join(dir, outFilename)];
    }), operators_1.concatMap(([src, dst]) => {
        console.log('Process "%s" to "%s"', src, dst);
        return compiler_1.compileTrie(src, dst).then(() => src);
    }))
        .forEach(name => console.log(`Complete.`));
});
program.parse(process.argv);
if (!process.argv.slice(2).length) {
    program.help();
}
//# sourceMappingURL=app.js.map