"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const gensequence_1 = require("gensequence");
const iterableToStream_1 = require("hunspell-reader/dist/iterableToStream");
const util_1 = require("hunspell-reader/dist/util");
var cspell_lib_1 = require("cspell-lib");
exports.writeToFile = cspell_lib_1.writeToFile;
exports.writeToFileRx = cspell_lib_1.writeToFileRx;
exports.writeToFileRxP = cspell_lib_1.writeToFileRxP;
function writeSeqToFile(seq, outFile) {
    return new Promise((resolve, reject) => {
        let resolved = false;
        const out = outFile ? fs.createWriteStream(outFile) : process.stdout;
        const bufferedSeq = gensequence_1.genSequence(util_1.batch(seq, 500)).map(batch => batch.join(''));
        const dataStream = iterableToStream_1.iterableToStream(bufferedSeq);
        const fileStream = dataStream.pipe(out);
        const endEvents = ['finish', 'close', 'end'];
        function resolvePromise() {
            if (!resolved) {
                resolved = true;
                resolve();
            }
        }
        const endHandler = () => {
            cleanupStreams();
            setTimeout(resolvePromise, 10);
        };
        const errorHandler = (e) => {
            cleanupStreams();
            reject(e);
        };
        listenToStreams();
        function listenToStreams() {
            endEvents.forEach(event => fileStream.addListener(event, endHandler));
            fileStream.addListener('error', errorHandler);
            dataStream.addListener('end', endHandler);
        }
        function cleanupStreams() {
            endEvents.forEach(event => fileStream.removeListener(event, endHandler));
            fileStream.removeListener('error', errorHandler);
            dataStream.removeListener('end', endHandler);
        }
    });
}
exports.writeSeqToFile = writeSeqToFile;
//# sourceMappingURL=fileWriter.js.map